
/*
* Copyright (c) 2008, Mike Giles
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * The name of Mike Giles may not be used to endorse or promote products
*       derived from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY Mike Giles ''AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL Mike Giles BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//
// this routine is used by both GPU and CPU routines to calculate
// the coefficients for the discrete finite difference equations
//

__host__ __device__ void params(int i, int j, int k, int I, int J, int K,
                                REAL dt, REAL *x, REAL *dx, REAL *xmin, REAL *dxi, 
                                REAL *src, REAL *u, REAL *sig, REAL *beta, REAL *consts)
{
  x[0] = xmin[0] + i*dx[0];
  x[1] = xmin[1] + j*dx[1];
  x[2] = xmin[2] + k*dx[2];

  model(x,src,u,sig,beta, consts);

  if (i>0 && i<I-1 && j>0 && j<J-1)
    beta[2] = 0.5*dt*dxi[0]*dxi[1]*beta[2]*sig[0]*sig[1];
  else
    beta[2] = 0.0;

  if (j>0 && j<J-1 && k>0 && k<K-1)
    beta[0] = 0.5*dt*dxi[1]*dxi[2]*beta[0]*sig[1]*sig[2];
  else
    beta[0] = 0.0;

  if (k>0 && k<K-1 && i>0 && i<I-1)
    beta[1] = 0.5*dt*dxi[2]*dxi[0]*beta[1]*sig[2]*sig[0];
  else
    beta[1] = 0.0;

  u[0]   = u[0]*dx[0];
  sig[0] = sig[0]*sig[0];
  if      (i==0   ||  u[0]>sig[0]) sig[0] =  u[0];
  else if (i==I-1 || -u[0]>sig[0]) sig[0] = -u[0];

  u[1]   = u[1]*dx[1];
  sig[1] = sig[1]*sig[1];
  if      (j==0   ||  u[1]>sig[1]) sig[1] =  u[1];
  else if (j==J-1 || -u[1]>sig[1]) sig[1] = -u[1];

  u[2]   = u[2]*dx[2];
  sig[2] = sig[2]*sig[2];
  if      (k==0   ||  u[2]>sig[2]) sig[2] =  u[2];
  else if (k==K-1 || -u[2]>sig[2]) sig[2] = -u[2];

  src[0] = dt*src[0];
  u[0]   = 0.5*dt*dxi[0]*dxi[0]*u[0];
  sig[0] = 0.5*dt*dxi[0]*dxi[0]*sig[0];

  src[1] = dt*src[1];
  u[1]   = 0.5*dt*dxi[1]*dxi[1]*u[1];
  sig[1] = 0.5*dt*dxi[1]*dxi[1]*sig[1];

  src[2] = dt*src[2];
  u[2]   = 0.5*dt*dxi[2]*dxi[2]*u[2];
  sig[2] = 0.5*dt*dxi[2]*dxi[2]*sig[2];
}

